# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: log.py
@date: 2022/08/28 10:53
"""
import logging
import os
import time


def single_instance(func):
    d = {}
    def __wap(name="app"):
        if name in d:
            return d[name]
        else:
            d[name] = func(name)
        return d[name]
    return __wap


@single_instance
class Logger():
    __file_name = None

    def __init__(self,name="app"):
        self.get_logger(name)
        self.get_haddler()
    @property
    def __log_file(self):
        if self.__file_name is None:
            # 把时间戳转换成时间元组
            time_tuple = time.localtime(time.time())
            # 把是时间元组，转换成时间字符串
            time_str = time.strftime("%Y%m%d",time_tuple)
            # 如果日志文件的文件夹不存在，则创建
            if not os.path.exists(os.path.join(os.getcwd(),"logs")):
                os.makedirs(os.path.join(os.getcwd(),"logs"))

            __class__.__file_name = os.path.join(os.getcwd(),"logs",f"{time_str}-")
        return self.__file_name

    def get_logger(self,name):
        # 创建日志记录器
        self.logger = logging.getLogger(name)
        # 设置日志记录器的级别
        self.logger.setLevel(level=logging.DEBUG)  # 设置日志收集的最低级别

    def get_haddler(self):
        '''
        创建日志收集器
        :return:
        '''
        # 创建formatter
        formater = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(module)s - %(lineno)d : %(message)s")
        # 创建一个debug级别的日志处理器
        handler_debug = logging.FileHandler(f"{self.__log_file}debug.log") # os.getcwd() 获取项目运行时路径
        handler_debug.setLevel(level=logging.DEBUG) # 设置日志处理的级别
        handler_debug.setFormatter(formater) # 绑定日志格式化器
        # 创建一个error级别的日志处理器
        handler_error = logging.FileHandler(f"{self.__log_file}error.log")
        handler_error.setLevel(level=logging.ERROR)
        handler_error.setFormatter(formater)
        # 和日志收集器绑定
        self.logger.addHandler(handler_debug)
        self.logger.addHandler(handler_error)

    def debug(self,msg):
        self.logger.debug(msg)

    def info(self,msg):
        self.logger.info(msg)

    def warning(self,msg):
        self.logger.warning(msg)

    def error(self,msg):
        self.logger.error(msg)

    def critical(self,msg):
        self.logger.critical(msg)

if __name__ == '__main__':
    Logger("aaa").info("aaaa")
    Logger("bbb").error("error")
    Logger().debug("debug")
    Logger().warning("warning")



# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: post_process_methods.py
@date: 2022/09/18 11:01
"""
# 后置关键字
import random

import jsonpath

from common.custom_exception import KeysExcutorFailed
from tools.log import Logger


def post_process(func):
    def __warp(send_request,*args,**kwargs):
        # 如果base_response属性中的值位None，说明该请求还未发出去或者是发送失败了。后置动作关键字，不可以被运行
        if send_request.base_response is None:
            Logger().error(f"该关键字，必须使用在后置动作中")
            raise KeysExcutorFailed
        res = func(send_request,*args,**kwargs)
        return res
    return __warp

# 令牌,$.data.list[9,2],1
# ("令牌","$.data.list[9","2]","1")
# *("令牌","$.data.list[9","2]","1")
# var_name = "令牌"   json_path = ("$.data.list[9","2]","1")
@post_process
def json_excutor(send_request,var_name,*json_path):
    # 获取接口响应数据
    response = send_request.base_response.response_body_dict
    # json_path（元组）:   json_path 索引号     索引号一定是一个数字，jsonpath，不可能是以数字结尾
    index = 0
    if json_path[-1].strip().isdigit(): # 如果元组中，最后一个元素，是一个数字。说明入参中，包含索引号
        index = int(json_path[-1].strip())
        json_path = json_path[:-1]
    json_path = ','.join(json_path) # 把元组，拼接成字符串
    res = jsonpath.jsonpath(response,json_path) # 根据jsonpath字符串，从字典中，提取到指定数据，存入列表中。res
    if index == 0: # 如果索引为0，则从jsonpath提取结果列表中，随机获取一个数据，存入变量
        s = random.choice(res)
        Logger().debug(f"json提取器从响应中提取到了提取到数据: {s}，存入变量：{var_name}")
        send_request.local_vars[var_name] = s
    elif len(res) >= index:
        Logger().debug(f"从响应中提取到了提取到数据:{res[index-1]}，存入变量：{var_name}")
        send_request.local_vars[var_name] = res[index-1]
    else:
        Logger().error(f"json提取器中，索引长度不能超过结果最大值。索引：{index}")
        raise KeysExcutorFailed




# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: keys_driver.py
@date: 2022/09/04 13:13
"""
from common.custom_exception import KeysExcutorFailed
from common.key_map import key_maps
from tools.log import Logger


class KeysDriver():

    def __init__(self,send_request,value):
        """
        包含关键字的字符串
        :param value:
        """
        self.send_request = send_request
        self.value = value
        self.key_replace()

    def keys_parser(self,value):
        '''
        关键字特征：$__开头  )$结尾
        $__PHONE()$
        $__RandomString(6,abcdefghijklmn)$
        $__RandomInt(10,20)$
        $__RandomString($__RandomInt(10,20)$,abcdefgh$ijklmn)$
        :param value:
        :return: 返回关键字组成的列表。普通关键字，直接放到列表中。嵌套关键字，把最外层的关键字放到列表中
        '''
        length = len(value) # 字符串的长度
        __keys = [] # 存放关键字的列表
        start = -1 # 关键字的起始索引，如果是-1则表示，未遇到过关键字
        flag = 0 # 作为关键字出现时，开始和结束是否是成对出现的标志
        for i in range(length): # 使用索引去遍历整个字符串
            if value[i] == "$":  # 说明，可能遇到了关键字的开始或者是结束标志
                # 判断关键字的起始
                if i + 2 < length and value[i:i+3] == "$__":
                    flag -= 1 # 遇到关键字起始，flag值减去1
                    # 记录一下，关键字的起始索引
                    if start < 0: # 如果start的值为-1.说明是第一次遇到关键字的起始索引（可能会涉及到关键字嵌套的问题）
                        start = i
                elif i-1 >= 0 and value[i-1:i+1] == ")$": # 判断关键字的结束
                    flag += 1 # 遇到关键字的结束，flag值就加上1
                    if flag == 0 and start >= 0: # flag等于0的时候，说明关键字起始和结束标记成对
                        __keys.append(value[start: i+1])
                        start = -1 # 重置关键字开始的索引
        return __keys # 返回关键字列表

    def key_run(self,key):
        '''
        关键字嵌套
        :param key:
        :return:
        '''
        # 判断关键字中，是否存在嵌套关系
        if "$__" in key[3:]:
            # 先处理内层关键字
            inner_keys = self.keys_parser(key[3:-1])
            for i_k in inner_keys:
                res = self.key_run(i_k) # 使用递归实现，内层关键字多层嵌套执行
                if res is not None and isinstance(res, str):
                    key = key.replace(i_k,res,1) # 使用内层关键字的执行结果，替换掉对应的内层关键字
        key_name = key[3:key.find("(")].strip().upper() # 获取关键字的名称
        key_args = key[key.find("(")+1:-2] # 获取关键字参数

        # 获取对应关键字方法的名字
        try:
            if key_args == "":
                return key_maps[key_name](self.send_request)
            else:
                return key_maps[key_name](self.send_request,*key_args.split(","))
        except KeyError:
            Logger().error(f"关键字名称：{key_name} 未定义")
            # print(f"关键字名称：{key_name} 未定义")
            return
        except KeysExcutorFailed:
            Logger().error(f"关键字{key},执行失败")
            # print(f"关键字{key},执行失败")
            return
        except AssertionError:
            raise

    def key_replace(self):
        # 解析数据中的关键字
        keys = self.keys_parser(self.value)
        for k in keys:
            res = self.key_run(k)  # 所有的关键字，执行结果必须是字符串。
            if res is not None and isinstance(res,str): # 如果返回值，不为空，则实现替换
                self.value = self.value.replace(k,res,1) # 使用关键字的执行结果，原字符串中对应的关键字






if __name__ == '__main__':

    s = "$__RandomString(8,012345$6789abcdefghijklmn)$需要用到$__RandomString(13,abcdefgh$ijklmn)$随机数据：$__RandomString(8,012345$6789abcdefghijklmn)$：用完了$__RandomString(8,012345$6789abcdefghijklmn)$"
    kd = KeysDriver(s)
    print(kd.value)




# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: utils.py
@date: 2022/08/28 15:51
"""
import json
# 尝试把一个字符串，转成python字典。字符串仅支持python字典的语法
def str_to_dict(s):
    if s.strip() == "":
        return None
    try:
        return eval(s)
    except:
        return s
# 尝试把json字符串转成python字典，字符串仅支持json的语法
def json_to_dict(s):
    if s.strip() == "":
        return None
    try:
        return json.loads(s)
    except:
        return s

def headers_to_dict(s):
    """
    请求头键值对，转字典格式
    :param s:
    :return:
    """
    d = {}
    lines = s.split("\n")
    for l in lines:
        line = l.split(":",1)
        if len(line) == 2:
            d[line[0]] = line[1].strip()
    return d

def params_to_dict(s):
    '''
    查询参数键值对，转字典格式
    :param s:
    :return:
    '''
    d = {}
    lines = s.split("&")
    for l in lines:
        line = l.split("=",1)
        if len(line) == 2:
            d[line[0]] = line[1].strip()
    return d



#
def dict_to_headers(d):
    return '\n'.join([f"{k}: {v}" for k,v in d.items()])


# if __name__ == '__main__':
#     s = '''Host: api.xuepl.com.cn:28019
# Connection: keep-alive
#
# '''
#     print(headers_to_dict(s))
# if __name__ == '__main__':
#     s = '''{'token':"$__GETVAR(令牌)$"}'''
#     res = str_to_dict(s)
#     print(res)
#
#     print(type(res))



# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: mysql_tools.py
@date: 2022/08/28 14:18
"""

import pymysql


class MySQLTool():
    def __init__(self, host, user, password, db, port=3306, charset="utf8"):
        self.host = host
        self.user = user
        self.password = password
        self.db = db
        self.port = port
        self.charset = charset

    def __connect(self):
        # 建立数据库链接
        self.__conn = pymysql.connect(
            host=self.host,
            user=self.user,
            password=self.password,
            db=self.db,
            port=self.port,
            charset=self.charset,
            cursorclass=pymysql.cursors.DictCursor
        )
        # 创建游标
        self.__cursor = self.__conn.cursor()

    def __enter__(self):
        '''
        进入上下文管理器的入口方法
        :return:
        '''
        self.__connect()
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        '''
        上下文管理器的退出方法
        :param exc_type:
        :param exc_val:
        :param exc_tb:
        :return:
        '''
        self.__close()

    def query(self, sql):
        '''
        查询
        :param sql: sql语句
        :return:
        '''
        self.__cursor.execute(sql)  # 执行sql语句
        res = self.__cursor.fetchall()  # 获取所有的查询结果
        return res

    def update(self, sql):
        '''
        增删改
        :param sql:sql语句
        :return:
        '''
        effect_row = self.__cursor.execute(sql)  # 执行sql语句
        self.__conn.commit()  # 增删改必须要提交，不然不生效
        return effect_row

    def __close(self):
        self.__cursor.close()  # 关闭游标
        self.__conn.close()  # 关闭数据库链接

if __name__ == '__main__':

    with MySQLTool(host="api.xuepl.com.cn",password="SongLin2021",user="root",db="mall") as db:
        print(db.update("select login_name,password_md5 from tb_newbee_mall_user limit 10;"))
        # print(db.update("update tb_newbee_mall_user set password_md5='e10adc3949ba59abbe56e057f20f883e' where user_id=17;"))



# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: assert_methods.py
@date: 2022/09/18 16:01
"""
import jsonpath

from common.custom_exception import KeysExcutorFailed
from tools.log import Logger
from tools.utils import json_to_dict


def assert_process(func):
    def __warp(send_request,*args,**kwargs):
        # 如果base_response属性中的值位None，说明该请求还未发出去或者是发送失败了。后置动作关键字，不可以被运行
        if send_request.base_response is None:
            Logger().error(f"该关键字，必须使用在断言动作中")
            raise KeysExcutorFailed
        res = func(send_request,*args,**kwargs)
        return res
    return __warp

# json断言
@assert_process
def json_assert(send_request,expect,*json_path):
    json_path = ",".join(json_path)
    # 获取响应数据
    response = send_request.base_response.response_body_dict
    # 使用jsonpath提取对应的数据
    res = jsonpath.jsonpath(response,json_path)# 列表
    actual = res[0]
    try:
        assert expect == actual,f"断言失败，jsonpath对应的数据：{actual} 不等于预期结果: {expect}"
    except:
        Logger().error(f"断言失败，{json_path}对应的数据：{actual} 不等于预期结果: {expect}")
        raise

# 响应断言
@assert_process
def respose_assert(send_request,*expect):
    # 获取响应数据
    response = send_request.base_response.response_body_text
    try:
        assert expect in response,f"断言失败，响应正文中不包含预期结果: {expect}"
    except:
        Logger().error(f"断言失败，响应正文中不包含预期结果: {expect}")
        raise

    pass


# 批量断言
@assert_process
def batch_assert(send_request,*expect):
    '''
    {"message":"SUCESS","resultCode":200}
    :param send_request:
    :param expect:
    :return:
    '''
    expect = ",".join(expect)
    expect = json_to_dict(expect)
    if not isinstance(expect,dict):
        Logger().error(f"批量断言中预期结果只能是json字符串：{expect}")
        assert False,f"批量断言中预期结果只能是json字符串：{expect}"
    response = send_request.base_response.response_body_dict
    compare_dict(response,expect)


def compare_dict(actual,expect):
    if not isinstance(expect,actual):
        Logger().error(f"批量断言中预期结果和响应格式不同")
        assert False, f"批量断言中预期结果只能是json字符串"
    if isinstance(expect,dict):
        for k in expect:
            compare_dict(actual[k], expect[k])
    elif isinstance(expect,list):
        for i in range(len(expect)):
            compare_dict(actual[i], expect[i])
    else:
        try:
            assert actual == expect,f"实际结果：{actual}和预期结果：{expect}不相等"
        except:
            Logger().error(f"实际结果：{actual}和预期结果：{expect}不相等")
            raise





# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: base_send_request.py
@date: 2022/08/28 16:42
"""
import json

import allure
import requests

from common.base_request import BaseRequest
from common.base_response import BaseResponse
from common.keys_driver import KeysDriver
from tools.utils import json_to_dict


class BaseSendRequest():
    base_request = None
    base_response = None
    def __init__(self,base_request):
        self.base_request = base_request
    def send_request(self):
        r = requests.request(
            method=self.base_request.method,
            url=self.base_request.url,
            json=self.base_request.json,
            data=self.base_request.data,
            headers=self.base_request.headers,
            files=self.base_request.files,
            params=self.base_request.params
        )
        self.base_response = BaseResponse(r)

class SendRequest(BaseSendRequest):
    local_vars = {}  # 相当于jmeter的变量池，存放代码中，通过关键字获取到的数据
    def __init__(self,case_dict):
        """
        用例数据组成的字典
        """
        # 获取前置操作数据
        self.pre_processers = case_dict["pre_processers"]
        # 获取请求相关信息
        base_request = BaseRequest()
        request_dict = {}
        request_dict["method"] =  case_dict["method"]
        request_dict["app"] =  case_dict["app"]
        request_dict["url"] =  case_dict["url"]
        request_dict["headers"] =  case_dict["headers"]
        request_dict["params"] =  case_dict["params"]
        request_dict["data"] =  case_dict["data"]
        request_dict["json"] =  case_dict["json"]
        request_dict["files"] =  case_dict["files"]
        self.request_dict = request_dict
        super().__init__(base_request)
        # 获取后置操作
        self.post_processers = case_dict["post_processers"]
        # 获取断言
        self.assert_data = case_dict["assert"]
        # 获取allure报告数据
        # d = {}
        # d["feature"] = case_dict["feature"]
        # d["story"] = case_dict["story"]
        # self.allure_data = d
        allure_list = ["feature","story"]
        self.allure_data = {k:case_dict[k] for k in case_dict if k in allure_list }

    def send_request(self):
        '''
        发送请求
        :return:
        '''
        # 添加allure报告信息
        for k in self.allure_data:
            getattr(allure.dynamic,k)(self.allure_data[k])
        # 第一步、前置动作（关键字）
        KeysDriver(self,self.pre_processers)
        # 第二步、请求发送
        # 关键字替换
        self.request_dict = json_to_dict(KeysDriver(self,json.dumps(self.request_dict,ensure_ascii=False)).value)
        # 初始化base_request
        self.base_request.url= (self.request_dict["app"],self.request_dict.pop("url")) # 先把url赋值给base__request对象
        for k,v in self.request_dict.items():
            setattr(self.base_request,k,v)
        # 关键字替换完成之后，把请求发送出去
        super().send_request()
        # 第三步、后置动作（关键字）
        KeysDriver(self,self.post_processers)
        # 第四步、断言（关键字）
        KeysDriver(self,self.assert_data)



# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: random_data.py
@date: 2022/09/04 14:32
"""

# 定义方法和关键字RandomString绑定的方法
import random

from faker import Faker

fake = Faker("zh_CN") # 默认是英文的，初始化成中文的

def random_str(send_request,length,content):
    res = random.choices(content,k=int(length))
    return "".join(res)

# 定义生成随机数的方法
def random_int(send_request,start,end): # 参数都是字符串
    res = random.randint(int(start),int(end))
    return str(res) # 返回值也需要是一个字符串

def random_phone(send_request):
    return f"{fake.phone_number()}"


# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: base_response.py
@date: 2022/08/28 16:55
"""
import os

from common.descriptions import ConfigDesc
from tools.log import Logger
from tools.utils import dict_to_headers


class BaseResponse():
    config = ConfigDesc()

    def __init__(self,response):
        self.response = response
        self.__print_log()

    @property
    def status_code(self):
        return self.response.status_code

    @property
    def response_body_text(self):
        return self.response.text

    @property
    def response_body_dict(self):
        return self.response.json()
    @property
    def response_body_content(self):
        return self.response.content

    def save_to_file(self,file_path):
        file_path = os.path.join(self.config.get_root_path(),file_path)
        with open(file_path,"wb") as f:
            f.write(self.response_body_content)
    @property
    def response_headers(self):
        return dict_to_headers(self.response.headers)

    @property
    def request_url(self):
        return self.response.request.url

    @property
    def request_method(self):
        return self.response.request.method

    @property
    def request_headers(self):
        return dict_to_headers(self.response.request.headers)

    @property
    def request_body(self):
        return self.response.request.body or ""

    def __print_log(self):
        # 打印请求和响应报文至日志
        Logger().debug(f"""
--------------------------请求报文-----------------------------
{self.request_method} {self.request_url}
{self.request_headers}

{self.request_body}""")

        Logger().debug(f"""
--------------------------响应报文-----------------------------
{self.status_code}
{self.response_headers}

{self.response_body_text}""")


# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: key_map.py
@date: 2022/09/04 14:32
"""
from tools.assert_methods import json_assert, respose_assert, batch_assert
from tools.common_methods import get_var, mysql_tools, save_file
from tools.post_process_methods import json_excutor
from tools.random_data import random_str, random_int, random_phone

# 添加关键字和对应方法之间的绑定关系
key_maps = {
    "RANDOMSTRING":random_str,
    "RANDOMINT":random_int,
    "PHONE":random_phone,
    "JSONEXCUTOR":json_excutor,
    "GETVAR":get_var,
    "MYSQL":mysql_tools,
    "JSONASSERT":json_assert,
    "RESPONSEASSERT":respose_assert,
    "BATCHASSERT":batch_assert,
    "SAVEFILE":save_file,
}



# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: descriptions.py
@date: 2022/08/28 15:35
"""
import json
import os

from tools.config import Config
from tools.log import Logger
from tools.utils import headers_to_dict, params_to_dict, str_to_dict, json_to_dict


class BaseDesc():
    __config = None
    @property
    def config(self):
        if self.__config is None:
            __class__.__config = Config()
        return self.__config

class ConfigDesc(BaseDesc):
    def __set__(self, instance, value):
        pass
    def __get__(self, instance, owner):
        return self.config


class MethodDesc():
    __value = None
    def __get__(self, instance, owner):
        return self.__value

    def __set__(self, instance, value):
        self.__value = value.strip()

class UrlDesc(BaseDesc):
    __value = None
    __app = None
    def __get__(self, instance, owner):
        base_url = self.config.get_url(self.__app)
        return f"{base_url.rstrip('/')}/{self.__value.lstrip('/')}"

    def __set__(self, instance, value):
        '''
        value 必须是一个元组 (应用名,接口地址)
        :param instance:
        :param value:
        :return:
        '''
        self.__app = value[0]
        self.__value = value[1]


class HeadersDesc():
    __value = None
    def __set__(self, instance, value):
        value = str_to_dict(value) # 尝试把一个字典类型的字符串，转成一个字典
        if isinstance(value,str):
            self.__value = headers_to_dict(value)
        else:
            self.__value = value

    def __get__(self, instance, owner):
        return self.__value


class ParamsDesc():
    __value = None
    def __set__(self, instance, value):
        value = str_to_dict(value)  # 尝试把一个字典类型的字符串，转成一个字典
        if isinstance(value, str):
            self.__value = params_to_dict(value)
        else:
            self.__value = value


    def __get__(self, instance, owner):
        return self.__value

class JsonDesc():
    __value = None
    def __set__(self, instance, value):
        if isinstance(value,str):
            self.__value = json_to_dict(value)
        else:
            self.__value = value

    def __get__(self, instance, owner):
        return self.__value


class DataDesc():
    __value = None
    def __set__(self, instance, value):
        value = str_to_dict(value)  # 尝试把一个字典类型的字符串，转成一个字典
        if isinstance(value,str) and "=" in value:
            self.__value = params_to_dict(value)
        else:
            self.__value = value

    def __get__(self, instance, owner):
        return self.__value

class FilesDesc(BaseDesc):
    __value = None
    def __get__(self, instance, owner):
        if isinstance(self.__value,list) or  isinstance(self.__value,tuple):
            res = []
            for l in self.__value:
                file_path = os.path.join(self.config.get_root_path(),l[1])
                try:
                    res.append((l[0],open(file_path,'rb')))
                except:
                    Logger().error(f"待上传文件路径不存在：{file_path}")
                    raise
            return res
        else:
            return self.__value

    def __set__(self, instance, value):
        '''
    [("关键字":"文件路径")]
    '''
        self.__value = str_to_dict(value)  # 尝试把一个字典类型的字符串，转成一个字典



# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: config.py
@date: 2022/08/28 13:27
"""
import configparser
import os
from tools.log import Logger

class Config():
    config = None

    def __init__(self):
        if self.config is None:
            __class__.config = configparser.ConfigParser() # 初始化解析器
            self.fille_path = os.path.join(os.getcwd(),f"{os.environ.get('env')}.ini")
            Logger().info(f"加载配置文件：{self.fille_path}")
            self.config.read(self.fille_path, encoding="utf-8")



    def get_url(self,app):
        try:
            return self.config.get("url",app)
        except:
            Logger().error(f"配置文件{self.fille_path}中，不存在对应的值。section: url,option: {app}")
            raise

    def get_db(self,app):
        try:
            db_config = dict(self.config.items(f"{app}_db"))
            db_config["port"] = int(db_config["port"])
            return db_config
        except:
            Logger().error(f"配置文件{self.fille_path}中，不存在对应的值。section: url,option: {app}")
            raise
    def get_root_path(self):
        root_path = self.config.get("base","root_path")
        root_path = root_path or os.getcwd()
        return root_path





# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: base_request.py
@date: 2022/08/28 16:32
"""
from common.descriptions import UrlDesc, MethodDesc, ParamsDesc, DataDesc, JsonDesc, HeadersDesc, FilesDesc, ConfigDesc


class BaseRequest():
    app = MethodDesc()
    url = UrlDesc()
    method = MethodDesc()
    params = ParamsDesc()
    data = DataDesc()
    json = JsonDesc()
    headers = HeadersDesc()
    files = FilesDesc()
    config = ConfigDesc()





# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: excel_tools.py
@date: 2022/09/04 15:58
"""
import os

import xlrd

from tools.config import Config

'''
给定一个excel的相对路径，
1、使用root_path 和excel的相对路径进行拼接
2、读取excel所有sheet中的数据，按照顺序，放入一个列表中
3、每条用用例数据，都是一个字典
'''
def read_excel(file_path):
    root_path = Config().get_root_path() # 获取root_path
    # root_path = "E:\\workspace\\edu\\auto_api_2207B"
    file_path = os.path.join(root_path,file_path) # 获取excel文件的绝对路径
    excel = xlrd.open_workbook(file_path) # 打开excel
    sheets = excel.sheets() # 获取到excel中，所有的sheet
    case_list = []
    ids = []
    for s in sheets: # 遍历每个一个sheet
        rows = s.nrows # 获取每个sheet中，有数据的行数
        if rows < 3:
            continue
        keys = s.row_values(1) # 读取excel中，第二行数据，作为字典的key
        for n in range(2,rows):
            row = s.row_values(n) # 把数据列，每行的数据，读出来
            case = dict(zip(keys,row)) # 把每行内容，转换成一个python字典
            if case["is_run"].strip() == "否" or case["title"] == "": # 如果用例不需要运行，则不放入列表中
                continue
            ids.append(case["title"]) # 用例标题，加入ids列表中
            case_list.append(case)# 用例数据放入case_list列表中
    return ids,case_list

def scan_excels(file_path):
    excel_files = []
    files = os.listdir(file_path)
    for f in files:
        file = os.path.join(file_path,f)
        if os.path.isdir(file) and f not in [".pytest_cache","venv",".idea",".git","__pycach__"]:
            res = scan_excels(file)
            excel_files.extend(res)
        elif os.path.isfile(file) and file.endswith(".xls"):
            excel_files.append(file)
    return excel_files


def get_cases():
    root_path = Config().get_root_path()
    excel_files = scan_excels(root_path)
    ids = []
    cases=[]
    for f in excel_files:
        i,cs = read_excel(f)
        ids.extend(i)
        cases.extend(cs)
    return ids,cases


if __name__ == '__main__':
    print(read_excel("ceshi.xls"))





# -*- coding:utf-8 -*-
"""
@author: xuepl
@file: common_methods.py
@date: 2022/09/18 11:48
"""
import os

import jsonpath

from common.custom_exception import KeysExcutorFailed
from tools.log import Logger
from tools.mysql_tools import MySQLTool


def get_var(send_request,var_name):
    '''
    令牌
    用户信息[0].login_name
    用户信息.login_name
    用户信息[0]['login_name']
    用户信息[0]["login_name"]
    :param send_request:
    :param var_name:
    :return:
    '''
    json_path = None
    if "." in var_name or "[" in var_name:
        one = var_name.find("[")
        two = var_name.find(".")
        if one == -1:
            index = two
        elif two == -1:
            index = one
        else:
            index = min(one,two)
        name = var_name[:index]
        json_path = "$" + var_name[index:]
    else:
        name = var_name
    local_vars = send_request.local_vars
    if name not in local_vars:
        Logger().error(f"变量名错误：{var_name}")
        raise KeysExcutorFailed
    value = local_vars[name]
    if json_path is None:
        return value
    else:
        return jsonpath.jsonpath(value,json_path)[0]

# 用户信息,mall,select login_name,password_md5 from tb_newbee_mall_user limit 10;
# *("用户信息","mall","select login_name","password_md5 from tb_newbee_mall_user limit 10;")
# app = "mall" args = ("用户信息","select login_name","password_md5 from tb_newbee_mall_user limit 10;")
def mysql_tools(send_request,app,*args):
    var_name=None
    sql = None
    one = args[0].strip()
    if len(one) > 6 and one[:6] in ["DELETE","UPDATE","INSERT","SELECT"]:
        sql = ",".join(args)
    else:
        var_name=args[0]
        sql = ",".join(args[1:])
    db_config = send_request.base_request.config.get_db(app)
    if sql[:6].upper() == "SELECT" and var_name is not None:
        with MySQLTool(**db_config) as db:
            res = db.query(sql)
            Logger().debug(f"sql查询成功：{res}")
            send_request.local_vars[var_name] = res
    else:
        with MySQLTool(**db_config) as db:
            db.update(sql)

# 把响应正文的数据，写入到文件中
def save_file(send_request,filepath):
    root_path = send_request.base_request.config.get_root_path()
    dir_name,file_name = os.path.split(filepath) # 获取文件夹路径和文件名字
    dir_path = os.path.join(root_path,dir_name) # 获取文件夹路径
    os.makedirs(dir_path) # 创建文件夹
    filepath = os.path.join(root_path,filepath)
    with open(filepath,"rb") as f:
        f.write(send_request.base_response.response_body_content)


